/// <reference types="jquery" />
/// <reference types="bootstrap" />
import { SourceFile, WorkerError, Segment, FileData } from "../common/workertypes";
import { CodeAnalyzer } from "../common/analysis";
import { ProbeRecorder } from "../common/recorder";
import * as pixed from "./pixeleditor";
export interface ProjectView {
    createDiv(parent: HTMLElement): HTMLElement;
    setVisible?(showing: boolean): void;
    refresh(moveCursor: boolean): void;
    tick?(): void;
    getPath?(): string;
    getValue?(): string;
    setText?(text: string): void;
    insertText?(text: string): void;
    getCursorPC?(): number;
    getSourceFile?(): SourceFile;
    setGutterBytes?(line: number, s: string): void;
    markErrors?(errors: WorkerError[]): void;
    clearErrors?(): void;
    setTimingResult?(result: CodeAnalyzer): void;
    recreateOnResize?: boolean;
    undoStep?(): void;
}
export declare class SourceEditor implements ProjectView {
    constructor(path: string, mode: string);
    path: string;
    mode: string;
    editor: any;
    dirtylisting: boolean;
    sourcefile: SourceFile;
    currentDebugLine: number;
    errormsgs: any[];
    errorwidgets: any[];
    inspectWidget: any;
    createDiv(parent: HTMLElement): HTMLDivElement;
    newEditor(parent: HTMLElement, isAsmOverride?: boolean): void;
    setupEditor(): void;
    inspect(ident: string): void;
    setText(text: string): void;
    insertText(text: string): void;
    replaceSelection(start: number, end: number, text: string): void;
    getValue(): string;
    getPath(): string;
    addErrorMarker(line: number, msg: string): void;
    addErrorLine(line: number, msg: string): void;
    expandErrors(): void;
    markErrors(errors: WorkerError[]): void;
    clearErrors(): void;
    getSourceFile(): SourceFile;
    updateListing(): void;
    setGutter(type: string, line: number, text: string): void;
    setGutterBytes(line: number, s: string): void;
    setTimingResult(result: CodeAnalyzer): void;
    setCurrentLine(line: number, moveCursor: boolean): void;
    clearCurrentLine(): void;
    getActiveLine(): number;
    refreshDebugState(moveCursor: boolean): void;
    refreshListing(): void;
    refresh(moveCursor: boolean): void;
    getLine(line: number): any;
    getCurrentLine(): number;
    getCursorPC(): number;
    undoStep(): void;
}
export declare class DisassemblerView implements ProjectView {
    disasmview: any;
    getDisasmView(): any;
    createDiv(parent: HTMLElement): HTMLDivElement;
    newEditor(parent: HTMLElement): void;
    refresh(moveCursor: boolean): void;
    getCursorPC(): number;
}
export declare class ListingView extends DisassemblerView implements ProjectView {
    assemblyfile: SourceFile;
    path: string;
    constructor(lstfn: string);
    refreshListing(): void;
    refresh(moveCursor: boolean): void;
}
export declare class MemoryView implements ProjectView {
    memorylist: any;
    dumplines: any;
    maindiv: HTMLElement;
    static IGNORE_SYMS: {
        s__INITIALIZER: boolean;
        _color_prom: boolean;
    };
    recreateOnResize: boolean;
    totalRows: number;
    createDiv(parent: HTMLElement): HTMLDivElement;
    showMemoryWindow(workspace: HTMLElement, parent: HTMLElement): void;
    scrollToAddress(addr: number): void;
    refresh(): void;
    tick(): void;
    getMemoryLineAt(row: number): string;
    readAddress(n: number): number;
    getDumpLineAt(line: number): string;
    getDumpLines(): any;
    getMemorySegment(a: number): string;
    findMemoryWindowLine(a: number): number;
}
export declare class VRAMMemoryView extends MemoryView {
    totalRows: number;
    readAddress(n: number): number;
    getMemorySegment(a: number): string;
    getDumpLines(): any;
}
export declare class BinaryFileView implements ProjectView {
    memorylist: any;
    maindiv: HTMLElement;
    path: string;
    data: Uint8Array;
    recreateOnResize: boolean;
    constructor(path: string, data: Uint8Array);
    createDiv(parent: HTMLElement): HTMLDivElement;
    showMemoryWindow(workspace: HTMLElement, parent: HTMLElement): void;
    getMemoryLineAt(row: number): string;
    refresh(): void;
    getPath(): string;
}
export declare class MemoryMapView implements ProjectView {
    maindiv: JQuery;
    createDiv(parent: HTMLElement): HTMLElement;
    addSegment(seg: Segment): void;
    refresh(): void;
}
declare abstract class ProbeViewBase {
    probe: ProbeRecorder;
    maindiv: HTMLElement;
    canvas: HTMLCanvasElement;
    ctx: CanvasRenderingContext2D;
    tooldiv: HTMLElement;
    recreateOnResize: boolean;
    createCanvas(parent: HTMLElement, width: number, height: number): HTMLDivElement;
    addr2str(addr: number): string;
    initCanvas(): void;
    showTooltip(s: string): void;
    getTooltipText(x: number, y: number): string;
    setVisible(showing: boolean): void;
    clear(): void;
    redraw(eventfn: (op: any, addr: any, col: any, row: any) => void): void;
    tick(): void;
    abstract drawEvent(op: any, addr: any, col: any, row: any): any;
}
declare abstract class ProbeBitmapViewBase extends ProbeViewBase {
    imageData: ImageData;
    datau32: Uint32Array;
    recreateOnResize: boolean;
    createDiv(parent: HTMLElement): HTMLDivElement;
    initCanvas(): void;
    getTooltipText(x: number, y: number): string;
    opToString(op: number, addr?: number): string;
    refresh(): void;
    tick(): void;
    clear(): void;
    getOpRGB(op: number): number;
}
export declare class AddressHeatMapView extends ProbeBitmapViewBase implements ProjectView {
    createDiv(parent: HTMLElement): HTMLDivElement;
    clear(): void;
    drawEvent(op: any, addr: any, col: any, row: any): void;
    getTooltipText(x: number, y: number): string;
}
export declare class RasterPCHeatMapView extends ProbeBitmapViewBase implements ProjectView {
    drawEvent(op: any, addr: any, col: any, row: any): void;
}
export declare class RasterStackMapView extends ProbeBitmapViewBase implements ProjectView {
    pcstack: any[];
    pushed: boolean;
    color: number;
    root: {};
    drawEvent(op: any, addr: any, col: any, row: any): void;
}
export declare class AssetEditorView implements ProjectView, pixed.EditorContext {
    maindiv: JQuery;
    cureditordiv: JQuery;
    cureditelem: JQuery;
    cureditnode: pixed.PixNode;
    rootnodes: pixed.PixNode[];
    deferrednodes: pixed.PixNode[];
    createDiv(parent: HTMLElement): HTMLElement;
    clearAssets(): void;
    registerAsset(type: string, node: pixed.PixNode, deferred: number): void;
    getPalettes(matchlen: number): pixed.SelectablePalette[];
    getTilemaps(matchlen: number): pixed.SelectableTilemap[];
    isEditing(): boolean;
    getCurrentEditNode(): pixed.PixNode;
    setCurrentEditor(div: JQuery, editing: JQuery, node: pixed.PixNode): void;
    scanFileTextForAssets(id: string, data: string): any[];
    addPaletteEditorViews(parentdiv: JQuery, pal2rgb: pixed.PaletteFormatToRGB, callback: any): void;
    addPixelEditor(parentdiv: JQuery, firstnode: pixed.PixNode, fmt: pixed.PixelEditorImageFormat): void;
    addPaletteEditor(parentdiv: JQuery, firstnode: pixed.PixNode, palfmt?: any): void;
    ensureFileDiv(fileid: string): JQuery<HTMLElement>;
    refreshAssetsInFile(fileid: string, data: FileData): number;
    getFileDivId(id: string): string;
    refresh(moveCursor: boolean): void;
    setVisible?(showing: boolean): void;
}
export {};
